#!/usr/bin/env bash 
##################################################
# SolarWinds APM .NET Agent Diagnostics Script
# FILE:         agent-diagnostic-logs.sh
# VERSION:      1.0.0
# DESCRIPTION:  SolarWinds APM .NET Agent script for collection of information and log files
# BUGS:         technicalsupport@solarwinds.com
# COPYRIGHT:    2021 SolarWinds Worldwide, LLC 
# LICENSE:      Apache 2.0; http://www.apache.org/licenses/LICENSE-2.0
# ORGANIZATION: http://solarwinds.com
#################################################

PROGNAME=$(basename $0)
WD="$(pwd)"
SCRIPT_ROOT_DIR="$( cd -P "$( dirname "$source" )" && pwd )"
echo "Script directory: ${SCRIPT_ROOT_DIR}"

export LOG_DIR=${WD}
mkdir -p ${LOG_DIR}
echo "Log directory: ${LOG_DIR}"

now()
{
  echo $(date +'%Y-%m-%d-%H_%M')
}

# Redirect all output to the log file

LOG_FILE="${LOG_DIR}/${PROGNAME}.`now`.log"
rm -rf ${LOG_FILE}
exec >  >(tee -ia ${LOG_FILE})
exec 2> >(tee -ia ${LOG_FILE} >&2)

# Truth values

__IS_TRUE=1
__IS_FALSE=0

__check_agent_home_env()
{
   if [ -z "${SW_APM_HOME_DOTNET}" ]; then
        # SW_APM_HOME_DOTNET is not defined
        echo $__IS_FALSE
   else
        # SW_APM_HOME_DOTNET is defined
        echo $__IS_TRUE
   fi
}
__HAS_AGENT_HOME_ENV=$(__check_agent_home_env)

# SOLARWINDS environment variables

__PRODUCT_NAME="SolarWinds"
__PRODUCT_NAME_UPPER="SOLARWINDS"
__PRODUCT_NAME_LOWER="solarwinds"
__SERVICE_KEY_ENV=${SW_APM_SERVICE_KEY:-""}
__HOME_APM_DOTNET_ENV=${SW_APM_HOME_DOTNET:-""}
__CONFIG_ENV=${SW_APM_CONFIG:-""}
__PROFILER_NAME="solarwinds_apm_profiler.so"
__ENV_PREFIX="SW_APM"
__CONFIG_FILE_NAME=solarwinds_apm.config
__PROFILER_CLASS_ID="{AAAA7777-11FE-4F94-8EDA-312C0EDF4141}"

__LOG_DIR="/var/tmp/${__PRODUCT_NAME_LOWER}"
__AZURE_LOG_DIR="${HOME}/LogFiles/${__PRODUCT_NAME}"
__HOME_DIR=${__HOME_APM_DOTNET_ENV}
__PROFILER=${__HOME_DIR}/${__PROFILER_NAME}
__CONFIG_FILE=${__HOME_DIR}/${__CONFIG_FILE_NAME}
__AGENT_DISPLAY_NAME="${__PRODUCT_NAME} APM .NET Agent"

# Archive settings

__OUTPUT_DIR=${WD}
__ARCHIVE_NAME="${__PRODUCT_NAME_LOWER}_agent_logs.`now`.tgz"

# Return error codes

readonly SUCCESS=0
readonly FAILURE=1

# Sizes

KILO=1024
let MEGA=$KILO*$KILO
let GIGA=$MEGA*$KILO
let TERA=$GIGA*$KILO

# Generate warning when trying to add a big file to archive

BIG_FILE_SIZE=100M

__error() 
{
    printf "* ERROR: %s\n" "$*" 1>&2
}

__info()
{
    printf "*  INFO: %s\n" "$*"
}

__warn() 
{
    printf "*  WARN: %s\n" "$*"
}


exists()
{
    command -v "$1" >/dev/null 2>&1
}

__file_exists()
{
    local check_file=${1}
    if [ -f "${check_file}" ]; then
        echo $__IS_TRUE
    else
        echo $__IS_FALSE
    fi
}

__dir_exists()
{
    local check_directory=${1}
    if [ -d "${check_directory}" ]; then
        echo $__IS_TRUE
    else
        echo $__IS_FALSE
    fi
}
__AGENT_IS_INSTALLED=$(__dir_exists "${__HOME_DIR}")

__file_size()
{
    local file=${1}
    if [ -f "${file}" ]; then
        size=$((du --apparent-size --block-size=1 "$file" 2>/dev/null \
                || gdu --apparent-size --block-size=1 "$file" 2>/dev/null \
                || find "$file" -printf "%s" 2>/dev/null \
                || gfind "$file" -printf "%s" 2>/dev/null \
                || stat --printf="%s" "$file" 2>/dev/null \
                || stat -f%z "$file" 2>/dev/null \
                || wc -c <"$file" 2>/dev/null \
               ) | awk '{print $1}')
        result=$?
        if [ $result -eq 0 ]
        then
            echo $size
        fi
    fi
    echo 0
}

__format_filesize()
{
    local size=${1}
    local formatted=""

    if [ "$size" -gt $TERA ]; then
        let size=$size/$TERA
        formatted="$size TB"
    elif [ "$size" -gt $GIGA ]; then
        let size=$size/$GIGA
        formatted="$size GB"
    elif [ "$size" -gt $MEGA ]; then
        let size=$size/$MEGA
        formatted="$size MB"
    elif [ "$size" -gt $KILO ]; then
        let size=$size/$KILO
        formatted="$size KB"
    elif [ "$size" -gt 0 ]; then
        formatted="$size B"
    fi

    echo "$formatted"
}

__failed()
{
    echo "###################################################"
    echo "An error occured during the collection of logs of ${__AGENT_DISPLAY_NAME}."
    echo "Please contact ${__PRODUCT_NAME} customer service for support at technicalsupport@${__PRODUCT_NAME_LOWER}.com."
    echo "###################################################"
    exit ${FAILURE}
}

__usage() 
{
cat <<EOU

  Usage:  ${0} [script options]

  Script Options:
    -h, --help                Show this help screen
    -o, --output <directory>  (Optional) Specifies output directory for the archive file. By default the archive file is created under current working directory.
EOU
}

__run_on_azure()
{
   local check_website_name=${WEBSITE_SITE_NAME:-""}
   if [ "${check_website_name}" == "" ]; then
       echo $__IS_FALSE
   else
       echo $__IS_TRUE
   fi
}
IS_AZURE=$(__run_on_azure)

# Parse command line arguments

while [ $# -gt 0 ]; do
    case ${1} in
    -h | --help)
        __usage
        exit
        ;;
    -o | --output)
        __OUTPUT_DIR=${2}
        shift 2
        continue
        ;;

    *)
    esac
    shift
done


__mask_service_key()
{
    local service_key=${1}
    local search=":"
    local show_len=4
    local pos_semicol=$(expr index ${service_key} ${search})
    local service_key_1=${service_key:0:${show_len}}
    local service_key_2=${service_key:${pos_semicol}-${show_len}-1:${show_len}}
    local service_name=${service_key:${pos_semicol}}
    local service_key_only=${service_key:0:${pos_semicol}}
    local pad_len=$((${#service_key_only}-2*${show_len}-1))
    local pad=$(printf '%*s' "${pad_len}")
    pad=${pad// /*}
    local hidden_service_key=${service_key_1}${pad}${service_key_2}":"${service_name}
    echo ${hidden_service_key}
}

__read_service_key()
{
    if [ "${__AGENT_IS_INSTALLED}" == "$__IS_TRUE" ]; then
        local check_file_config=$(__file_exists "${__CONFIG_FILE}")
        if [ "${check_file_config}" == "$__IS_FALSE" ]; then
            __warn "Unable to read service key from ${__CONFIG_FILE}. File not found."
            echo ""
        fi
        local config_service_key=$(sed -n -e '/\(key="ServiceKey"\)/p' "${__CONFIG_FILE}" | grep -oP '(value=").*(")' | grep -oP '(").*(")' | sed 's/["\n]//g')
        echo ${config_service_key}
   fi
}
__SERVICE_KEY_CONFIG_FILE=$(__read_service_key)

__read_log_file_location()
{
    if [ "${__AGENT_IS_INSTALLED}" == "$__IS_TRUE" ]; then
        local check_file_config=$(__file_exists "${__CONFIG_FILE}")
        if [ "${check_file_config}" == "$__IS_FALSE" ]; then
            __warn "Unable to read log file location from ${__CONFIG_FILE}. File not found."
            echo ""
        fi
        local log_file_location=$(sed -n -e '/\(key="LogFileLocation"\)/p' "${__CONFIG_FILE}" | grep -oP '(value=").*(")' | grep -oP '(").*(")' | sed 's/["\n]//g')
        echo ${log_file_location}
   fi
}
__LOG_FILE_LOCATION=$(__read_log_file_location)

__find_dotnet_version()
{
    local dotnet_runtime_dir='/usr/share/dotnet/shared/Microsoft.NETCore.App'
    local check_dotnet_runtime_dir=$(__dir_exists "${dotnet_runtime_dir}")
    local dotnet_runtime_version=""
    if [ "${check_dotnet_runtime_dir}" == "${__IS_TRUE}" ]; then
        for dir in ${dotnet_runtime_dir}/*
        do
          dir=${dir%*/}
          dotnet_runtime_version="${dir##*/}"
        done
    fi
    echo ${dotnet_runtime_version}
}
DOTNET_RUNTIME_VERSION=$(__find_dotnet_version)
DOTNET_RUNTIME_MAJOR_VERSION=${DOTNET_RUNTIME_VERSION%.*.*}

__check_install()
{
    # Check if agent home environment variable is set

    if [ "${__HAS_AGENT_HOME_ENV}" == "$__IS_FALSE" ]; then
        __error "Required ${__ENV_PREFIX}_HOME_DOTNET environment variable is not set. Unable to find installation directory."
        return 
    fi

    # Check if agent installation directoryi exists

    if [ "${__AGENT_IS_INSTALLED}" == "$__IS_FALSE" ]; then
        __error "Unable to find ${__HOME_DIR} installation directory. ${__AGENT_DISPLAY_NAME} is not installed."
        return 
    fi
    __info "${__AGENT_DISPLAY_NAME} is installed in ${__HOME_DIR}."

    local found_service_key=${__IS_FALSE}

    # Read service key from environment

    if [ -n "${__SERVICE_KEY_ENV}" ]; then
        local masked_service_key_env=$(__mask_service_key ${__SERVICE_KEY_ENV})
        __info "Service key set in ${__ENV_PREFIX}_SERVICE_KEY environment variable: ${masked_service_key_env}"
        found_service_key=${__IS_TRUE}
    else 
        __info "No service key set in ${__ENV_PREFIX}_SERVICE_KEY environment variable."
    fi

    # Read service key from configuration file

    if [ -n "${__SERVICE_KEY_CONFIG_FILE}" ]; then
        local masked_service_key=$(__mask_service_key ${__SERVICE_KEY_CONFIG_FILE})
        __info "Found ${masked_service_key} service key inside ${__CONFIG_FILE} ..."
        found_service_key=${__IS_TRUE}
    else
        __info "No service key found inside ${__CONFIG_FILE}."
    fi

    if [ "${found_service_key}" == "${__IS_FALSE}" ]; then
        __error "No service key set is in ${__ENV_PREFIX}_SERVICE_KEY environment variable or found in i${__CONFIG_FILE} configuration file."
    fi

    # Check .NET Core profiler environment variables

    local profiler_is_enabled=""
    if [ -n "${CORECLR_ENABLE_PROFILING}" ]; then
        if [ "${CORECLR_ENABLE_PROFILING}" == "1" ]; then
            profiler_is_enabled="enabled"
        else
            profiler_is_enabled="disabled"
        fi
    else
        __warn "Environment variable CORECLR_ENABLE_PROFILING is not set. To enable ${__AGENT_DISPLAY_NAME} profiler set CORECLR_ENABLE_PROFILING to 1".
    fi

    if [ -n "${CORECLR_PROFILER_PATH}" ]; then
        if [ "${CORECLR_PROFILER_PATH}" != "${__PROFILER}" ]; then
            __error "Environment variable CORECLR_PROFILER_PATH is set a wrong value ${CORECLR_PROFILER_PATH}. To enable ${__AGENT_DISPLAY_NAME} set CORECLR_PROFILER_PATH to ${__PROFILER}."
        fi
    else
        __warn "Environment variable CORECLR_PROFILER_PATH is not set. To enable ${__AGENT_DISPLAY_NAME} profiler set CORECLR_PROFILER_PATH to ${__PROFILER}."
    fi

    if [ -n "${CORECLR_PROFILER}" ]; then
        if [ "${CORECLR_PROFILER}" == "${__PROFILER_CLASS_ID}" ]; then
            __info "${__AGENT_DISPLAY_NAME} profiler is ${profiler_is_enabled}."
        else
            __error "Found a different .NET Core profiler ${CORECLR_PROFILER}. Profiler ${CORECLR_PROFILER_PATH} is ${profiler_is_enabled}. "
        fi
    else
        __warn "Environment variable CORECLR_PROFILER is not set. To enable ${__AGENT_DISPLAY_NAME} profiler you need to set CORECLR_PROFILER to ${__PROFILER_CLASS_ID}."
    fi
}

__display_info_env()
{
    local name=${1}
    local value=${2}
    __info "${name} = ${value}"
}

__display_environment()
{
    __info "-------------------------------------------------------------"
    __info "Global Environment Variables:"

    __display_info_env "${__ENV_PREFIX}_HOME_DOTNET" "${__HOME_APM_DOTNET_ENV}"
    __display_info_env "${__ENV_PREFIX}_CONFIG" "${__CONFIG_ENV}"
    __display_info_env "${__ENV_PREFIX}_SERVICE_KEY" "${__SERVICE_KEY_ENV}"

    __display_info_env "CORECLR_ENABLE_PROFILING" "${CORECLR_ENABLE_PROFILING}"
    __display_info_env "CORECLR_PROFILER" "${CORECLR_PROFILER}"
    __display_info_env "CORECLR_PROFILER_PATH" "${CORECLR_PROFILER_PATH}"
    __info "-------------------------------------------------------------"

}

__display_dotnet_info()
{
    __info "-------------------------------------------------------------"
    __info ".NET Core Runtime Information:"
    __info "Version: ${DOTNET_RUNTIME_VERSION}"
    CORECLR_ENABLE_PROFILING=0 dotnet --info
    __info "-------------------------------------------------------------"
}

__do_collect()
{
    local check_log_dir=$(__dir_exists ${__LOG_DIR})
    if [ "${check_log_dir}" == "${__IS_FALSE}" ]; then
        __warn "${__AGENT_DISPLAY_NAME} logging directory ${__LOG_DIR} doesn't exist. No agent log files to be collected."
        return
    fi

    local check_log_file_location_dir=$(__dir_exists ${__LOG_FILE_LOCATION})
    if [ "${check_log_file_location_dir}" == "${__IS_FALSE}" ]; then
        __warn "${__AGENT_DISPLAY_NAME} custom logging directory ${__LOG_FILE_LOCATION} doesn't exist. No agent log files to be collected."
    fi

    echo Creating package ${__ARCHIVE_NAME} ...

    # Check file sizes into logging directory
    
    IFS=$'\n' x=($(find ${__LOG_DIR} -type f -size +${BIG_FILE_SIZE}))
    len=${#x[@]}
    if [ "$len" -gt 0 ]; then
        __warn "Found files with size bigger than ${BIG_FILE_SIZE} in ${__LOG_DIR}."
        for file in "${x[@]}"; do
            __warn "Size for $file is greater than ${BIG_FILE_SIZE}.";
        done
    fi

    # Check file sizes into custom logging directory
    
    if [ "${check_log_file_location_dir}" == "${__IS_TRUE}" ]; then
        IFS=$'\n' x=($(find ${__LOG_FILE_LOCATION} -type f -size +${BIG_FILE_SIZE}))
        len=${#x[@]}
        if [ "$len" -gt 0 ]; then
            __warn "Found files with size bigger than ${BIG_FILE_SIZE} in ${__LOG_FILE_LOCATION}."
            for file in "${x[@]}"; do
                __warn "Size for $file is greater than ${BIG_FILE_SIZE}.";
            done
        fi
    fi

    # Copy log file to logging directory to be archived

    cp ${LOG_FILE} ${__LOG_DIR}

    # Copy log files from custom log directory to logging directory to be archived
    
    if [ "${check_log_file_location_dir}" == "${__IS_TRUE}" ]; then
       cp ${__LOG_FILE_LOCATION}/* ${__LOG_DIR}
    fi

    local package_file=${__OUTPUT_DIR}/${__ARCHIVE_NAME}

    pushd ${__LOG_DIR}
    tar -zcvf ${package_file} .
    popd

    # Remove log file from logging directory

    logfilename=$(basename ${LOG_FILE})
    rm -f ${__LOG_DIR}/${logfilename}

    # Check that archive file was created

    if [ ! -f "${package_file}" ]; then
        __error "Failed to create archive ${package_file}"
        exit ${FAILURE}
    fi

    echo ${package_file} created.
}

__display_information()
{
    local package_file=${__OUTPUT_DIR}/${__ARCHIVE_NAME}
    local archive_file_size_bytes=$(__file_size "${package_file}")
    local archive_file_size_formatted=$(__format_filesize ${archive_file_size_bytes})

    local log_file_size_bytes=$(__file_size "${LOG_FILE}")
    local log_file_size_formatted=$(__format_filesize ${log_file_size_bytes})
    __info "-------------------------------------------------------------"
    __info "Results:"
    __info "  Archive : $package_file Size: ${archive_file_size_formatted}"
    __info "  Log file: ${LOG_FILE} Size: ${log_file_size_formatted} "
    __info ""
    __info "Please email archive and log file to us:"
    __info "   email: technicalsupport@${__PRODUCT_NAME_LOWER}.com"
    __info "-------------------------------------------------------------"
}

__check_parameters()
{
    local check_dir=$(__dir_exists ${__OUTPUT_DIR})
    if [ "${check_dir}" == "${__IS_FALSE}" ]; then
        __error "Directory ${__OUTPUT_DIR} specified by --output parameter doesn't exist."
        exit ${FAILURE}
    fi
}

main()
{
    # On Azure platform, log directory is to a different location.

    if [ "$IS_AZURE" == "${__IS_TRUE}" ]; then
        __LOG_DIR=${__AZURE_LOG_DIR}
    fi

    # Check if log directory exists to collect logs

    local check_log_dir=$(__dir_exists ${__LOG_DIR})
    if [ "${check_log_dir}" == "${__IS_FALSE}" ]; then
        __warn "${__AGENT_DISPLAY_NAME} logging directory ${__LOG_DIR} doesn't exist. No agent log files to be collected."
    fi

    # Check if custom log directory exists to collect logs

    local check_log_file_location_dir=$(__dir_exists ${__LOG_FILE_LOCATION})
    if [ "${check_log_file_location_dir}" == "${__IS_FALSE}" ]; then
        __warn "${__AGENT_DISPLAY_NAME} custom logging directory ${__LOG_FILE_LOCATION} doesn't exist. No agent log files to be collected."
    fi

    __info "-------------------------------------------------------------"
    __info "Running script to collect ${__AGENT_DISPLAY_NAME} information..."
    __info ""
    __info "Computer name: `hostname` "
    __info "Current date : `echo $(date +'%Y-%m-%d %H:%M')` "
    __info ""
    __info "For detailed instructions, FAQ, and full documentation:"
    __info "   http://docs.${__PRODUCT_NAME_LOWER}.com"
    __info ""
    __info "If you have any questions or problems, please contact us:"
    __info "   email: technicalsupport@${__PRODUCT_NAME_LOWER}.com"
    __info "-------------------------------------------------------------"

    __check_parameters
    __display_environment
    __display_dotnet_info
    __check_install
    __do_collect
    __display_information
}

main

echo Log file: ${LOG_FILE}
echo ${PROGNAME} done.

exit ${SUCCESS}
